import gleam/dict.{type Dict}
import gleam/dynamic/decode.{type Decoder}
import models/trade_symbol.{type TradeSymbol}
import utils/api.{type ApiResponse}
import utils/auth.{NoAuth}

pub type GetSupplyChainResponse {
  GetSupplyChainResponse(
    export_to_import_map: Dict(TradeSymbol, List(TradeSymbol)),
  )
}

fn get_supply_chain_response_decoder() -> Decoder(GetSupplyChainResponse) {
  use export_to_import_map <- decode.field(
    "exportToImportMap",
    decode.dict(trade_symbol.decoder(), decode.list(trade_symbol.decoder())),
  )
  decode.success(GetSupplyChainResponse(export_to_import_map:))
}

pub fn get_supply_chain() -> ApiResponse(GetSupplyChainResponse) {
  let request = api.get(NoAuth, "/market/supply-chain")
  use response <- api.try_send(request)
  echo response.body
  case response.status {
    200 ->
      api.parse_data_response(response, get_supply_chain_response_decoder())
    _ -> api.parse_error_response(response)
  }
}
